/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: RezControl.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public partial class RezControl : UserControl
    {
        private RezPropertyPage parentPage; 
        public RezControl()
        {
            InitializeComponent();
            this.tbRezName.Focus();
            parentPage = null;
        }

        public RezControl(RezPropertyPage parentPage)
        {
            InitializeComponent();
            this.tbRezName.Focus();
            this.parentPage = parentPage;
            this.tbRezIp.ReadOnly = true;
        }

        private void RezChanged(object sender, EventArgs e)
        {
            //if control is being used by propertey page
            if (this.parentPage != null)
            {
                this.parentPage.Dirty = true;
            }
        }

        public Boolean GetData(out DhcpReservation rez)
        {
            if (!this.MyValidate())
            {
                rez = null;
                return false;
            }

            rez = new DhcpReservation();
            rez.Name = tbRezName.Text.Trim() ?? String.Empty;
            rez.ReservedIp = new DhcpIpAddress(tbRezIp.Text.Trim());
            rez.ReservedMac = new DhcpMacAddress(tbRezMac.Text.Trim());
            rez.Comment = tbRezDesc.Text.Trim() ?? String.Empty;

            if (rbDhcpType.Checked)
                rez.bAllowedClientTypes = DhcpSubnetClientType.Dhcp;
            else if (rbBootpType.Checked)
                rez.bAllowedClientTypes = DhcpSubnetClientType.Bootp;
            else
                rez.bAllowedClientTypes = DhcpSubnetClientType.Both;

            return true;
        }

        public void SetData(DhcpReservation rez)
        {
            this.tbRezName.Text = rez.Name;
            this.tbRezIp.Text = rez.ReservedIp.ToString();
            this.tbRezMac.Text = rez.ReservedMac.ToString();
            this.tbRezDesc.Text = rez.Comment;

            switch (rez.bAllowedClientTypes)
            {
                case DhcpSubnetClientType.Dhcp:
                    rbDhcpType.Checked = true;
                    break;
                case DhcpSubnetClientType.Bootp:
                    rbBootpType.Checked = true;
                    break;
                case DhcpSubnetClientType.Both:
                    rbBothType.Checked = true;
                    break;
            }

            if (this.parentPage != null)
            {
                this.parentPage.Dirty = false;
            }
        }

        public Boolean MyValidate()
        {
            String caption = "Input Error";
            //rez name
            if (tbRezName.Text.Trim().Length < 1)
            {
                MessageBox.Show(this.ParentForm, "Name field blank", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //Rez ip
            DhcpIpAddress rezIp;
            try
            {
                rezIp = new DhcpIpAddress(tbRezIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this.ParentForm, "Rez IP not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DhcpMacAddress rezMac;
            try
            {
                rezMac = new DhcpMacAddress(tbRezMac.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this.ParentForm, "Rez MAC not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DhcpIpAddress network, mask;
            if (this.parentPage != null)
            {
                network = ((RezScopeNode)((RezPropertyPage)this.parentPage).rezList.ScopeNode).SubnetNode.DhcpSubnet.Address;
                mask = ((RezScopeNode)((RezPropertyPage)this.parentPage).rezList.ScopeNode).SubnetNode.DhcpSubnet.Mask;
            }
            else
            {
                network = ((RezForm)this.ParentForm).rezNode.SubnetNode.DhcpSubnet.Address;
                mask = ((RezForm)this.ParentForm).rezNode.SubnetNode.DhcpSubnet.Mask;
            }

            //make sure rezIp is not network Ip
            if (rezIp.GetUIntAddress() == network.GetUIntAddress())
            {
                MessageBox.Show(this.ParentForm, "Start IP cannot equal Network Address", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //check rezIP is in network
            if ((rezIp.GetUIntAddress() & mask.GetUIntAddress()) != network.GetUIntAddress())
            {
                MessageBox.Show(this.ParentForm, "Reservation IP not in this subnet", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }
    }
}
