/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: OptionScopeNode.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;


using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public class OptionScopeNode : ScopeNode
    {
        public ServerScopeNode ServerNode;
        public DhcpOptionScope OptScope;
        public List<DhcpOptionValue> OptionValues;

        public OptionScopeNode(ServerScopeNode serverNode, DhcpOptionScope optScope)
        {
            
            this.OptScope = optScope;
            this.ServerNode = serverNode;

            this.DisplayName = optScope.type.ToString() + " Options";
            this.ImageIndex = this.SelectedImageIndex = 2;

            //List View
            MmcListViewDescription lvd = new MmcListViewDescription();
            lvd.DisplayName = this.DisplayName;
            lvd.ViewType = typeof(OptionListView);
            lvd.Options = MmcListViewOptions.SingleSelect;
            this.ViewDescriptions.Add(lvd);
            this.ViewDescriptions.DefaultIndex = 0;

            //verbs
            this.EnabledStandardVerbs = StandardVerbs.Refresh;

            //actions
            this.ActionsPaneItems.Add(new SyncAction("Set Option...", "Set Option", 2, "SetOption"));

            this.setOptions();
        
        }

        protected override void OnSyncAction(SyncAction action, SyncStatus status)
        {
            switch ((string)action.Tag)
            {
                case "SetOption":
                    this.ShowPropertySheet(this.DisplayName);
                    break;
            }
        }

        protected override void OnRefresh(AsyncStatus status)
        {
            //raise change event
            this.setOptions();
            this.RaiseOnChange();
        }

        protected override void OnAddPropertyPages(PropertyPageCollection propertyPageCollection)
        {
            propertyPageCollection.Add(new OptionPropertyPage(this));    
        }

        private void setOptions()
        {
            DhcpOptionValue[] optArray;
            this.OptionValues = new List<DhcpOptionValue>(5);
            try
            {
                if ((optArray = this.ServerNode.ProfileNode.DhcpOps.EnumOptionValues(
                    this.ServerNode.ServerIp, DhcpOptionClassType.Dhcp, this.OptScope)) != null)
                    this.OptionValues.AddRange(optArray);
                if ((optArray = this.ServerNode.ProfileNode.DhcpOps.EnumOptionValues(
                   this.ServerNode.ServerIp, DhcpOptionClassType.Bootp, this.OptScope)) != null)
                    this.OptionValues.AddRange(optArray);
                if ((optArray = this.ServerNode.ProfileNode.DhcpOps.EnumOptionValues(
                   this.ServerNode.ServerIp, DhcpOptionClassType.Routing, this.OptScope)) != null)
                    this.OptionValues.AddRange(optArray);
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Option Error", "Unable to query DHCP Options" + Environment.NewLine + "Error: " + e.Message);
                return;
            }
        }

        //Based on Platform SDK Example
        #region Event related methods  ------------------------------------------

        /// <summary>
        /// Declare the delegate type for the changed event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void ChangedDelegate(object sender, EventArgs ev);

        /// <summary>
        /// internal store of delegates needed when using event 'lock' syntax 
        /// </summary>
        private ChangedDelegate changedDelegate;

        /// <summary>
        /// Changed event to notify that the scope node has changed
        /// </summary>
        public event ChangedDelegate Changed
        {
            add
            {
                lock (this)
                    changedDelegate += value;
            }
            remove
            {
                lock (this)
                    changedDelegate -= value;
            }
        }

        /// <summary>
        /// Raises the changed event using the private delegate
        /// </summary>
        public void RaiseOnChange()
        {
            // safely invoke an event
            ChangedDelegate raiseChangedDelegate = changedDelegate;
            if (raiseChangedDelegate != null)
            {
                raiseChangedDelegate(this, new EventArgs());
            }
        }

        #endregion
    }
}
