/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: ServerScopeNode.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;
using Unf.Dhcp.ManagementConsole.DwsSec;

namespace Unf.Dhcp.ManagementConsole
{
    public class ServerScopeNode : ScopeNode
    {
        public ProfileScopeNode ProfileNode;
        public DwsOps.DhcpIpAddress ServerIp;

        public Dictionary<uint, DhcpOption> DhcpOptions = new Dictionary<uint, DhcpOption>(66);

        public ServerScopeNode(ProfileScopeNode profile, String serverIp)
        {
            this.DisplayName = "DHCP Server [" + serverIp + "]";
            this.ImageIndex = this.SelectedImageIndex = 6;
            
            this.ServerIp = new DwsOps.DhcpIpAddress(serverIp);
            ProfileNode = profile;


            //actions
            this.ActionsPaneItems.Add(new Action("Create Subnet...", "Create a Subnet", 7, "CreateSubnet"));

            //verbs
            this.EnabledStandardVerbs = StandardVerbs.Refresh | StandardVerbs.Properties | StandardVerbs.Delete;

            //list view
            this.SetMmcView();

            
        }

        protected override void OnExpand(AsyncStatus status)
        {
            base.OnExpand(status);

            //populate childern
            this.OnRefresh(null);

            //populate options
            this.GetDhcpOptions();
        }

        protected override void OnAction(Action action, AsyncStatus status)
        {
            switch ((string)action.Tag)
            {
                case "CreateSubnet":
                    {
                        SubnetForm subForm = new SubnetForm(this);

                        if (this.SnapIn.Console.ShowDialog(subForm) == DialogResult.OK)
                        {
                            //
                        }
                        break;
                    }
            }
        }

        protected override void OnAddPropertyPages(PropertyPageCollection propertyPageCollection)
        {
            propertyPageCollection.Add(new DnsPropertyPage(this));
        }

        protected override void OnDelete(SyncStatus status)
        {
            String msgText = "'" + this.DisplayName + "' will be removed from MMC view";
            if (SnapInUtil.PopUpConfirm(msgText) == DialogResult.OK)
            {
                this.Parent.Children.Remove(this);
                this.ProfileNode.SaveData.ServerIPs.Remove(this.ServerIp.ToString());
                this.SnapIn.IsModified = true;
            }
        }

        protected override void OnRefresh(AsyncStatus status)
        {
            this.Children.Clear();

            //add subnets
            DhcpSubnet[] subnets;
            try
            {
                subnets = ProfileNode.DhcpOps.EnumSubnets(this.ServerIp);
                if (subnets != null && subnets.Length != 0)
                {
                    foreach (DhcpSubnet subnet in subnets)
                        this.Children.Add(new SubnetScopeNode(this, subnet));
                }
                //add option node
                this.Children.Add(new OptionScopeNode(this, new DhcpOptionScope()));
            }
            catch
            {
                SnapInUtil.PopUpError("Unable to Connect",
                    "Unable to connect to DHCP Server: " + this.ServerIp + Environment.NewLine +
                    "Is server IP correct?");
                return;
            }

            

            //add permission node
            if (ProfileNode.isAdmin)
            {
                
                DhcpAzServerScope azScope = new DhcpAzServerScope();
                azScope.Server = new DwsSec.DhcpIpAddress(this.ServerIp.Ip);

                try
                {
                    DhcpAzAccount[] accounts = ProfileNode.DhcpSec.GetPermissions(azScope);
                    this.Children.Add(new PermissionsScopeNode(ProfileNode, azScope));
                }
                catch
                {
                    SnapInUtil.PopUpError("Unable to Connect",
                    "Unable to connect to DHCP Server: " + this.ServerIp + Environment.NewLine +
                    "Is server IP correct?");
                    return;
                }
            }
        }

        private void SetMmcView()
        {
            AddViewDescription("Default View", typeof(SubnetListView));
            ViewDescriptions.DefaultIndex = 0;  
        }

        private void GetDhcpOptions()
        {
            try
            {
                DhcpOption[] arr = ProfileNode.DhcpOps.EnumOptions(ServerIp, DhcpOptionClassType.Dhcp);
                foreach (DhcpOption opt in arr)
                    DhcpOptions.Add(opt.OptionId, opt);
            }
            catch { }
        }

        #region Switch Views...
        /// <summary>
        /// Adds a view description and set it as the default
        /// </summary>
        /// <param name="displayName"></param>
        /// <param name="viewType"></param>
        public void AddViewDescription(string displayName, Type viewType)
        {
            // add the view description (display name is required)
            //List View
            MmcListViewDescription lvd = new MmcListViewDescription();
            lvd.DisplayName = displayName;
            lvd.ViewType = viewType;
            if (viewType == typeof(SearchListView))
                lvd.Options = MmcListViewOptions.ExcludeScopeNodes | MmcListViewOptions.HideSelection;
            this.ViewDescriptions.Add(lvd);

        }

        /// <summary>
        /// Removes a message view description 
        /// </summary>
        /// <param name="removeIndex"></param>
        public void RemoveViewDescriptionAt(int removeIndex)
        {
            // remove the description
            this.ViewDescriptions.RemoveAt(removeIndex);
        }

        /// <summary>
        /// Swaps a new view for the existing. 
        /// (NOTE: expects that only a single view is already defined)
        /// </summary>
        /// <param name="currentView"></param>
        /// <param name="newViewDisplayName"></param>
        /// <param name="newViewType"></param>
        public void SwapView(Microsoft.ManagementConsole.View currentView, string newViewDisplayName, Type newViewType)
        {
            // remove the existing view description 
            this.RemoveViewDescriptionAt(0);

            // add the new view description
            this.AddViewDescription(newViewDisplayName, newViewType);

            // set the new view as the default
            this.ViewDescriptions.DefaultIndex = 0;

            // trigger MMC to show the new default view 
            currentView.SelectScopeNode(this);
        }
        #endregion
    }
}
