/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: RezListView.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public class RezListView : MmcListView
    {
        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            // Create a set of columns for use in the list view
            // Define the default column title
            this.Columns[0].Title = "Reserved IP";
            this.Columns[0].SetWidth(125);

            // Add detail column
            this.Columns.Add(new MmcListViewColumn("Name", 100));
            this.Columns.Add(new MmcListViewColumn("Hardware Address", 125));
            this.Columns.Add(new MmcListViewColumn("Type", 75));
            this.Columns.Add(new MmcListViewColumn("Description", 175));

            //listen for scope event
            ((RezScopeNode)this.ScopeNode).Changed += new RezScopeNode.ChangedDelegate(OnScopeNodeChange);

            //allow selected node to be deleted
            this.SelectionData.EnabledStandardVerbs = StandardVerbs.Delete;

            this.Refresh();
        }

        protected override void OnDelete(SyncStatus status)
        {
             String msgText = "'" + ((ResultNode)this.SelectedNodes[0]).DisplayName + "' Reservation will be deleted";
             if (SnapInUtil.PopUpConfirm(msgText) == DialogResult.OK)
             {
                 SubnetScopeNode subnetNode = ((RezScopeNode)this.ScopeNode).SubnetNode;
                 RezResultNode node = (RezResultNode)this.SelectedNodes[0];
                 try
                 {
                     subnetNode.ProfileNode.DhcpOps.DeleteReservation(subnetNode.ServerIp, subnetNode.DhcpSubnet.Address, node.Reservation.ReservedIp);
                     this.ResultNodes.Remove(node);
                 }
                 catch (Exception e)
                 {
                     SnapInUtil.PopUpError("Reservation Error", "Unable to delete DHCP Reservation" + Environment.NewLine + "Error: " + e.Message);
                 }
             }
        }

        /// <summary>
        /// Cleanup. 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnShutdown(SyncStatus status)
        {
            //Stop listening for scope event
            ((RezScopeNode)this.ScopeNode).Changed -= new RezScopeNode.ChangedDelegate(OnScopeNodeChange);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnSelectionChanged(SyncStatus status)
        {
            if (this.SelectedNodes.Count == 0)
            {
                this.SelectionData.Clear();
            }
            else
            {
                this.SelectionData.Update(this.SelectedNodes[0], false, null, null);
                this.SelectionData.ActionsPaneItems.Clear();
                //special properties action
                this.SelectionData.ActionsPaneItems.Add(new Action("Properties", "Properties", -2, "Properties"));

            }
        }

        // <summary>
        /// Handle any change to the scope node. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OnScopeNodeChange(object sender, EventArgs ev)
        {
            this.Refresh();
        }

        /// <summary>
        /// 
        /// </summary>
        private void Refresh()
        {
            this.ResultNodes.Clear();

            SubnetScopeNode subnetNode = ((RezScopeNode)this.ScopeNode).SubnetNode;
            try
            {
                DhcpReservation[] rezs = subnetNode.ProfileNode.DhcpOps.EnumReservations(subnetNode.ServerIp, subnetNode.DhcpSubnet.Address);
                if (rezs == null)
                    rezs = new DhcpReservation[0];

                foreach (DhcpReservation rez in rezs)
                {
                    RezResultNode node = new RezResultNode(rez);
                    this.ResultNodes.Add(node);
                }
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Reservation Error", "Unable to query DHCP Reservations" + Environment.NewLine + "Error: " + e.Message);
            }
        }

        protected override void OnSelectionAction(Action action, AsyncStatus status)
        {
            switch ((String)action.Tag)
            {
                case "Properties":
                    ((RezResultNode)this.SelectedNodes[0]).SetOptionNode(((RezScopeNode)this.ScopeNode).SubnetNode);
                    this.SelectionData.ShowPropertySheet(this.SelectedNodes[0].DisplayName + " Reservation Properties");
                    break;
            }
        }

        /// <summary>
        /// OnAddPropertyPages is used to get the property pages to show.
        /// </summary>
        /// <param name="propertyPageCollection">property pages</param>
        protected override void OnAddPropertyPages(PropertyPageCollection propertyPageCollection)
        {
            if (this.SelectedNodes.Count == 1)
            {
                // add at least one property page relevant to the selection
                propertyPageCollection.Add(new RezPropertyPage(this));
                propertyPageCollection.Add(new DnsPropertyPage(((RezScopeNode)this.ScopeNode).SubnetNode, 
                    ((RezResultNode)this.SelectedNodes[0]).Reservation.ReservedIp));
                propertyPageCollection.Add(new OptionPropertyPage(((RezResultNode)this.SelectedNodes[0]).OptionNode));
            }
        }
    }
}
