/* 
 * DHCP Server Management Objects
 *
 *  File: AzServerOperations.cs
 *  Namespace: Unf.Dhcp.Smo.WebServices
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using Microsoft.Interop.Security.AzRoles;

using Unf.Dhcp.Smo;
using Unf.Dhcp.Smo.Operations;

namespace Unf.Dhcp.Smo.WebServices
{
    /// <summary>
    /// Authorization wrapper for DHCP operations found in <see cref="Unf.Dhcp.Smo.Operations.ServerOperations"/>.  This wrapper uses Microsoft's
    /// Authorization Manager framework (AzMan) to provide an authorization layer on DHCP server operation classes.
    /// </summary>
    /// <seealso href="http://msdn2.microsoft.com/en-us/library/aa480244.aspx">Developing Applications Using Windows Authorization Manager</seealso>
    /// <seealso href="http://technet2.microsoft.com/WindowsServer/en/library/72b55950-86cc-4c7f-8fbf-3063276cd0b61033.mspx?mfr=true">Role-Based Access Control for Multi-tier Applications Using Authorization Manager</seealso>
    /// <seealso href="http://msdn2.microsoft.com/en-us/library/aa375751(VS.85).aspx">Authorization Manager Interfaces</seealso>
    public class AzServerOperations
    {

        #region Subnet Ops...
        //requires limited on subnet
        /// <summary>
        /// Enumerate subnets on server
        /// </summary>
        /// <param name="server">Server IP address to enumerate</param>
        /// <permission>Requires internal limited role.  This role will be automatically assigned based on other roles within the subnet and/or IP range</permission>
        /// <remarks>This method implements security trimming.  User will only 'see' subnets they can access</remarks>
        /// <returns>A list of DhcpSubnets</returns>
        public static DhcpSubnet[] EnumSubnets(DhcpIpAddress server)
        {
            List<DhcpSubnet> allSubnets = ServerOperations.EnumSubnets(server);
            List<DhcpSubnet> trimedSubnets = new List<DhcpSubnet>();

            IAzClientContext2 ctx = DhcpSecurityOps.GetClientContext();
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;

            foreach (DhcpSubnet subnet in allSubnets)
            {
                azSubnet.Subnet = subnet.Address;
                if (azSubnet.AccessCheck("EnumSubnets", ctx, DhcpAzOps.Traverse))
                    trimedSubnets.Add(subnet);
            }

            return trimedSubnets.ToArray();
        }

        //requires create on server+
        /// <summary>
        /// Create a DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address</param>
        /// <param name="ipRange">Default address pool associated with subnet</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role at server scope</permission>
        /// <remarks>ipRange parameter can be of type <see cref="DhcpIpRange"/> or <see cref="BootpIpRange"/></remarks>
        public static void CreateSubnet(DhcpIpAddress server, DhcpSubnet network, DhcpIpRange ipRange)
        {
            DhcpAzServerScope azServer = new DhcpAzServerScope();
            azServer.Server = server;

            if (azServer.AccessCheck("CreateSubnet", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Create))
                ServerOperations.CreateSubnet(server, network, ipRange);
            else
                throw new UnauthorizedAccessException("CreateSubnet Access Denied");
        }

        //requires delete on server+
        /// <summary>
        /// Delete DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address identifying subnet</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role at server scope</permission>
        public static void DeleteSubnet(DhcpIpAddress server, DhcpIpAddress network)
        {
            DhcpAzServerScope azServer = new DhcpAzServerScope();
            azServer.Server = server;

            if (azServer.AccessCheck("DeleteSubnet", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Delete))
            {
                ServerOperations.DeleteSubnet(server, network);

                DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                azSubnet.Server = server;
                azSubnet.Subnet = network;
                DhcpSecurityOps.RemoveAzSubnetAzIpRanges(azSubnet);
            }
            else
                throw new UnauthorizedAccessException("DeleteSubnet Access Denied");
        }

        //requires update on subnet+
        /// <summary>
        /// Update DHCP subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet object to update</param>
        /// <remarks>Operation will update network.Name, network.Comment, and network.State on existing subnet matching network.Address</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        public static void UpdateSubnet(DhcpIpAddress server, DhcpSubnet network)
        {
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;
            azSubnet.Subnet = network.Address;

            if (azSubnet.AccessCheck("UpdateSubnet", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.UpdateSubnet(server, network);
            else
                throw new UnauthorizedAccessException("UpdateSubnet Access Denied");
            
        }

        //requires update on subnet+
        /// <summary>
        /// Update default DHCP subnet address pool (range)
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to update pool on</param>
        /// <param name="ipRange">New default address pool associated with subnet</param>
        /// <remarks>
        /// ipRange parameter can be of type <see cref="DhcpIpRange"/> or <see cref="BootpIpRange"/><br/>
        /// Removes current address pool and replaces with ipRange.  This can be used to change address pool client type (Dhcp, Bootp, or Both)
        /// </remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        public static void UpdateSubnetRange(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipRange)
        {
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;
            azSubnet.Subnet = network;

            if (azSubnet.AccessCheck("UpdateSubnetRange", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.UpdateSubnetRange(server, network, ipRange);
            else
                throw new UnauthorizedAccessException("UpdateSubnetRange Access Denied");
        }

        //requires update on subnet+
        /// <summary>
        /// Create a subnet exclusion address pool
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to create pool in</param>
        /// <param name="ipExRange">IP range use to created pool</param>
        /// <remarks>ipExRange.Type must equal <see cref="DhcpSubnetElementDataType.DhcpExcludedIpRanges"/></remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        public static void CreateSubnetExclusion(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipExRange)
        {
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;
            azSubnet.Subnet = network;

            if (azSubnet.AccessCheck("CreateSubnetExclusion", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.CreateSubnetExclusion(server, network, ipExRange);
            else
                throw new UnauthorizedAccessException("CreateSubnetExclusion Access Denied");
            
        }

        //requires update on subnet+
        /// <summary>
        /// Deletes a subnet exclusion address pool
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to delete pool from</param>
        /// <param name="ipExRange">IP range to delete</param>
        /// <remarks>ipExRange.Type must equal <see cref="DhcpSubnetElementDataType.DhcpExcludedIpRanges"/></remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role at subnet scope</permission>
        public static void DeleteSubnetExclusion(DhcpIpAddress server, DhcpIpAddress network, DhcpIpRange ipExRange)
        {
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;
            azSubnet.Subnet = network;

            if (azSubnet.AccessCheck("DeleteSubnetExclusion", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.DeleteSubnetExclusion(server, network, ipExRange);
            else
                throw new UnauthorizedAccessException("DeleteSubnetExclusion Access Denied");
            
        }

        //requires read on ipr+ 
        /// <summary>
        /// Search subnet for client leases matching terms
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <returns>A list of client leases matching terms</returns>
        /// <remarks>Client filter terms are .NET regex patterns<br/>
        /// This method implements security trimming.  User will only 'see' client leases their authorized to access
        /// </remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        public static DhcpClient[] SubnetFindAllClients(DhcpIpAddress server, DhcpIpAddress network, DhcpClientFilter terms)
        {
            List<DhcpClient> allClients = ServerOperations.SubnetFindAllClients(server, network, terms);
            List<DhcpClient> trimedClients = new List<DhcpClient>();

            IAzClientContext2 ctx = DhcpSecurityOps.GetClientContext();
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;

            foreach (DhcpClient client in allClients)
            {
                azIpRange.StartIp = client.IpAddress;
                if (azIpRange.AccessCheck("SubnetFindAllClients", ctx, DhcpAzOps.Read))
                    trimedClients.Add(client);
            }

            return trimedClients.ToArray();
        }

        //requires read on ipr+
        /// <summary>
        /// Search subnet for first client lease matching terms
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns<br/>
        /// This method implements security trimming.  User will only 'see' client leases their authorized to access
        /// </remarks>
        /// <returns>First matching client lease or null if no match</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        public static DhcpClient SubnetFindOneClient(DhcpIpAddress server, DhcpIpAddress network, DhcpClientFilter terms)
        {
            DhcpClient oneClient = ServerOperations.SubnetFindOneClient(server, network, terms);
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;
            azIpRange.StartIp = oneClient.IpAddress;

            if (azIpRange.AccessCheck("SubnetFindOneClient", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return oneClient;
            else
                return null;
        }

        //requires read on server+
        /// <summary>
        /// Search <b>all</b> subnets on server looking for client leases matching terms
        /// </summary>
        /// <param name="server">Server IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns
        /// </remarks>
        /// <returns>A list of client leases matching terms</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at server scope</permission>
        public static DhcpClient[] ServerFindAllClients(DhcpIpAddress server, DhcpClientFilter terms)
        {
            List<DhcpClient> allClients = ServerOperations.ServerFindAllClients(server, terms);

            IAzClientContext2 ctx = DhcpSecurityOps.GetClientContext();
            DhcpAzServerScope azServer = new DhcpAzServerScope();
            azServer.Server = server;

            if (azServer.AccessCheck("ServerFindAllClients", ctx, DhcpAzOps.Read))
                return allClients.ToArray();
            else
                throw new UnauthorizedAccessException("ServerFindAllClients Access Denied, Requires Read on Server");

        }

        //requires read on server+
        /// <summary>
        /// Search <b>all</b> subnets on server looking for first client lease matching terms
        /// </summary>
        /// <param name="server">Server IP address to search</param>
        /// <param name="terms">Client filter terms</param>
        /// <remarks>Client filter terms are .NET regex patterns
        /// </remarks>
        /// <returns>First matching client lease or null if no match</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at server scope</permission>
        public static DhcpClient ServerFindOneClient(DhcpIpAddress server, DhcpClientFilter terms)
        {
            DhcpClient oneClient = ServerOperations.ServerFindOneClient(server, terms);
            DhcpAzServerScope azServer = new DhcpAzServerScope();
            azServer.Server = server;

            if (azServer.AccessCheck("ServerFindOneClient", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return oneClient;
            else
                throw new UnauthorizedAccessException("ServerFindAllClients Access Denied, Requires Read on Server");
        }

        //requires read on subnet+
        /// <summary>
        /// Enumerate all address pools belonging to subnet
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address</param>
        /// <returns>An array of address pools.  The first pool will be the default address pool followed by zero or more exclusion pools</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role at subnet scope</permission>
        public static DhcpIpRange[] EnumPools(DhcpIpAddress server, DhcpIpAddress network)
        {
            DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
            azSubnet.Server = server;
            azSubnet.Subnet = network;

            if (azSubnet.AccessCheck("EnumPools", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return ServerOperations.EnumPools(server, network);
            else
                return null;
        }
        #endregion    

        #region Lease Ops...

        //requries ip+ read
        /// <summary>
        /// Enumerate all client leases on subnet
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to enumerate</param>
        /// <returns>A list of client leases belonging to subnet</returns>
        /// <remarks>This method implements security trimming.  User will only 'see' client leases their authorized to access</remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning IP address of client lease</permission>
        public static DhcpClient[] EnumLeases(DhcpIpAddress server, DhcpIpAddress network)
        {
            List<DhcpClient> allClients = ServerOperations.EnumLeases(server, network);
            List<DhcpClient> trimedClients = new List<DhcpClient>();

            IAzClientContext2 ctx = DhcpSecurityOps.GetClientContext();
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;

            foreach (DhcpClient client in allClients)
            {
                azIpRange.StartIp = client.IpAddress;
                if (azIpRange.AccessCheck("EnumLeases", ctx, DhcpAzOps.Read))
                    trimedClients.Add(client);
            }

            return trimedClients.ToArray();
        }

        //requires ip+ create
        /// <summary>
        /// Create a client lease on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="lease">Client lease info to create</param>
        /// <remarks>
        /// Client lease will be assigned to a existing subnet based on the binary AND operation of lease.IpAddress &amp; lease.SubnetMask
        /// </remarks>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning IP address of client lease</permission>
        public static void CreateLease(DhcpIpAddress server, DhcpClient lease)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = new DhcpIpAddress(lease.IpAddress.GetUIntAddress() & lease.SubnetMask.GetUIntAddress());
            azIpRange.StartIp = lease.IpAddress;

            if (azIpRange.AccessCheck("CreateLease", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Create))
                ServerOperations.CreateLease(server, lease);
            else
                throw new UnauthorizedAccessException("CreateLease Access Denied");
        }

        //requires ip+ update
        /// <summary>
        /// Update a client lease on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="lease">Client lease to match and update</param>
        /// <remarks>
        /// Client lease will be updated on a existing subnet based on the binary AND operation of lease.IpAddress &amp; lease.SubnetMask<br/>
        /// Operation will update lease.Mac, lease.Name, lease.Comment, and lease.LeaseExpires on existing lease matching lease.IpAddress and lease.SubnetMask
        /// </remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role on scope owning IP address of client lease</permission>
        public static void UpdateLease(DhcpIpAddress server, DhcpClient lease)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = new DhcpIpAddress(lease.IpAddress.GetUIntAddress() & lease.SubnetMask.GetUIntAddress()); ;
            azIpRange.StartIp = lease.IpAddress;

            if (azIpRange.AccessCheck("UpdateLease", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.UpdateLease(server, lease);
            else
                throw new UnauthorizedAccessException("UpdateLease Access Denied");
        }

        //requires ip+ delete
        /// <summary>
        /// Delete client lease from server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to delete lease from</param>
        /// <param name="leaseIp">Client lease IP address, used to identify and delete</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning IP address of client lease</permission>
        public static void DeleteLease(DhcpIpAddress server, DhcpIpAddress network, DhcpIpAddress leaseIp)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;
            azIpRange.StartIp = leaseIp;

            if (azIpRange.AccessCheck("DeleteLease", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Delete))
                ServerOperations.DeleteLease(server, leaseIp);
            else
                throw new UnauthorizedAccessException("DeleteLease Access Denied");
        }

        #endregion

        #region Rez Ops...
        //requires ip+ create
        /// <summary>
        /// Create a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to create reservation on</param>
        /// <param name="res">IP reservation info</param>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning reservation's IP address</permission>
        public static void CreateReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpReservation res)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;
            azIpRange.StartIp = res.ReservedIp;

            if (azIpRange.AccessCheck("CreateReservation", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Create))
                ServerOperations.CreateReservation(server, network, res);
            else
                throw new UnauthorizedAccessException("CreateReservation Access Denied");
        }

        //requires ip+ update
        /// <summary>
        /// Update a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to update reservation on</param>
        /// <param name="res">Updated IP reservation info</param>
        /// <remarks>Updates res.ReservedMac, res.bAllowedClientTypes, res.Name, and res.Comment based on match of res.ReservedIp</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least manager role on scope owning reservation's IP address</permission>
        public static void UpdateReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpReservation res)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;
            azIpRange.StartIp = res.ReservedIp;

            if (azIpRange.AccessCheck("UpdateReservation", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.UpdateReservation(server, network, res);
            else
                throw new UnauthorizedAccessException("UpdateReservation Access Denied");
        }

        //requires ip+ delete
        /// <summary>
        /// Delete a DHCP IP reservation on server
        /// </summary>
        /// <param name="server">server IP address</param>
        /// <param name="network">Subnet network IP address to delete reservation from</param>
        /// <param name="resIp">Reservation IP address to delete</param>
        /// <remarks>Lease associated with reservation will also be deleted</remarks>
        /// <permission cref="DhcpAzRole.Owner">Requires owner role on scope owning reservation's IP address</permission>
        public static void DeleteReservation(DhcpIpAddress server, DhcpIpAddress network, DhcpIpAddress resIp)
        {
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;
            azIpRange.StartIp = resIp;

            if (azIpRange.AccessCheck("DeleteReservation", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Delete))
                ServerOperations.DeleteReservation(server, network, resIp);
            else
                throw new UnauthorizedAccessException("DeleteReservation Access Denied");
        }

        //requires ip+ read
        /// <summary>
        /// Enumerate IP Reservations owned by subnet on server
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="network">Subnet network IP address to enumerate</param>
        /// <returns>A list of reservations on subnet</returns>
        /// <remarks>This method implements security trimming.  User will only 'see' reservations their authorized to access</remarks>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least auditor role on scope owning reservation's IP address</permission>
        public static DhcpReservation[] EnumReservations(DhcpIpAddress server, DhcpIpAddress network)
        {
            List<DhcpReservation> allRezz = ServerOperations.EnumReservations(server, network);
            List<DhcpReservation> trimedRezz = new List<DhcpReservation>();

            IAzClientContext2 ctx = DhcpSecurityOps.GetClientContext();
            DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
            azIpRange.Server = server;
            azIpRange.Subnet = network;

            foreach (DhcpReservation rez in allRezz)
            {
                azIpRange.StartIp = rez.ReservedIp;
                if (azIpRange.AccessCheck("EnumReservations", ctx, DhcpAzOps.Read))
                    trimedRezz.Add(rez);
            }

            return trimedRezz.ToArray();
        }
        #endregion

        #region Option Ops...

        //requires update on contexted based scope
        /// <summary>
        /// Sets an DHCP option's value based on a particular scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="value">DHCP option value</param>
        /// <remarks>This method overwrites any previous data set on value.OptionId</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least Manager role on scope owning option value</permission>
        public static void SetOptionValue(DhcpIpAddress server, DhcpOptionValue value)
        {
            DhcpAzScope contextScope = null;
            switch (value.OptionScope.type)
            {
                case DhcpOptionScopeType.Server:
                    DhcpAzServerScope azServer = new DhcpAzServerScope();
                    azServer.Server = server;
                    contextScope = azServer;
                    break;
                case DhcpOptionScopeType.Subnet:
                    DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                    azSubnet.Server = server;
                    azSubnet.Subnet = value.OptionScope.SubnetIp;
                    contextScope = azSubnet;
                    break;
                case DhcpOptionScopeType.Reservation:
                    DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
                    azIpRange.Server = server;
                    azIpRange.Subnet = value.OptionScope.SubnetIp;
                    azIpRange.StartIp = value.OptionScope.ReservationIp;
                    contextScope = azIpRange;
                    break;
            }

            if (contextScope.AccessCheck("SetOptionValue", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.SetOptionValue(server, value);
            else
                throw new UnauthorizedAccessException("SetOptionValue Access Denied");
        }

        //requires update on contexted based scope
        /// <summary>
        /// Removes a DHCP option's value
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="value">DHCP option value</param>
        /// <remarks>Completes remove all values from option.  Option becomes unset</remarks>
        /// <permission cref="DhcpAzRole.Manager">Requires at least Manager role on scope owning option value</permission>
        public static void RemoveOptionValue(DhcpIpAddress server, DhcpOptionValue value)
        {
            DhcpAzScope contextScope = null;
            switch (value.OptionScope.type)
            {
                case DhcpOptionScopeType.Server:
                    DhcpAzServerScope azServer = new DhcpAzServerScope();
                    azServer.Server = server;
                    contextScope = azServer;
                    break;
                case DhcpOptionScopeType.Subnet:
                    DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                    azSubnet.Server = server;
                    azSubnet.Subnet = value.OptionScope.SubnetIp;
                    contextScope = azSubnet;
                    break;
                case DhcpOptionScopeType.Reservation:
                    DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
                    azIpRange.Server = server;
                    azIpRange.Subnet = value.OptionScope.SubnetIp;
                    azIpRange.StartIp = value.OptionScope.ReservationIp;
                    contextScope = azIpRange;
                    break;
            }

            if (contextScope.AccessCheck("RemoveOptionValue", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Update))
                ServerOperations.RemoveOptionValue(server, value);
            else
                throw new UnauthorizedAccessException("RemoveOptionValue Access Denied");
        }

        //requires read on contexted based scope
        /// <summary>
        /// Enumerate all DHCP option values set in scope
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing</param>
        /// <param name="scope">Option scope (Server, Subnet, or Reservation) to enumerate</param>
        /// <returns>An array of DHCP option's values current set in scope</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least Auditor role on scope owning option values</permission>
        public static DhcpOptionValue[] EnumOptionValues(DhcpIpAddress server, DhcpOptionClassType classType, DhcpOptionScope scope)
        {
            DhcpAzScope contextScope = null;
            switch (scope.type)
            {
                case DhcpOptionScopeType.Server:
                    DhcpAzServerScope azServer = new DhcpAzServerScope();
                    azServer.Server = server;
                    contextScope = azServer;
                    break;
                case DhcpOptionScopeType.Subnet:
                    DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                    azSubnet.Server = server;
                    azSubnet.Subnet = scope.SubnetIp;
                    contextScope = azSubnet;
                    break;
                case DhcpOptionScopeType.Reservation:
                    DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
                    azIpRange.Server = server;
                    azIpRange.Subnet = scope.SubnetIp;
                    azIpRange.StartIp = scope.ReservationIp;
                    contextScope = azIpRange;
                    break;
            }

            if (contextScope.AccessCheck("EnumOptionValues", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return ServerOperations.EnumOptionValues(server, classType, scope);
            else
                return null;
        }

        //requires read on contexted based scope
        /// <summary>
        /// Retrieves a DHCP option's value 
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="optionId">DHCP option identifier (ex. 006 = DNS Servers)</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing)</param>
        /// <param name="scope">Get option value from scope (Server, Subnet, or Reservation)</param>
        /// <returns>A DHCP option value</returns>
        /// <permission cref="DhcpAzRole.Auditor">Requires at least Auditor role on scope owning option value</permission>
        public static DhcpOptionValue GetOptionValue(DhcpIpAddress server, UInt32 optionId, DhcpOptionClassType classType, DhcpOptionScope scope)
        {
            DhcpAzScope contextScope = null;
            switch (scope.type)
            {
                case DhcpOptionScopeType.Server:
                    DhcpAzServerScope azServer = new DhcpAzServerScope();
                    azServer.Server = server;
                    contextScope = azServer;
                    break;
                case DhcpOptionScopeType.Subnet:
                    DhcpAzSubnetScope azSubnet = new DhcpAzSubnetScope();
                    azSubnet.Server = server;
                    azSubnet.Subnet = scope.SubnetIp;
                    contextScope = azSubnet;
                    break;
                case DhcpOptionScopeType.Reservation:
                    DhcpAzIpRangeScope azIpRange = new DhcpAzIpRangeScope();
                    azIpRange.Server = server;
                    azIpRange.Subnet = scope.SubnetIp;
                    azIpRange.StartIp = scope.ReservationIp;
                    contextScope = azIpRange;
                    break;
            }

            if (contextScope.AccessCheck("GetOptionValue", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return ServerOperations.GetOptionValue(server, optionId, classType, scope);
            else
                return null;
        }

        //only requires access to DWS itself
        /// <summary>
        /// Enumerate all DHCP options available on server (global set of DHCP options)
        /// </summary>
        /// <param name="server">Server IP address</param>
        /// <param name="classType">DHCP option class type (Dhcp, Bootp, or Routing)</param>
        /// <returns>An array of global DHCP options available on server</returns>
        /// <remarks>Only Dhcp class types seems to return a set of possible DHCP options.  The other classes share these same DHCP options</remarks>
        /// <permission>Does not require authorization, only authentication</permission>
        public static DhcpOption[] EnumOptions(DhcpIpAddress server, DhcpOptionClassType classType)
        {
            //DhcpAzServerScope azServer = new DhcpAzServerScope();
            //azServer.Server = server;

            //if (azServer.AccessCheck("EnumOption", DhcpSecurityOps.GetClientContext(), DhcpAzOps.Read))
                return ServerOperations.EnumOptions(server, classType);
            //else
             //   return null;
        }

        #endregion

    }
}
